import { deepEqual } from "./deep.js";
export var LogLevel;
(function (LogLevel) {
    LogLevel[LogLevel["debug"] = 0] = "debug";
    LogLevel[LogLevel["info"] = 1] = "info";
    LogLevel[LogLevel["warn"] = 2] = "warn";
    LogLevel[LogLevel["error"] = 3] = "error";
    LogLevel[LogLevel["fatal"] = 4] = "fatal";
})(LogLevel || (LogLevel = {}));
export function LoggerConsoleHandler(level = LogLevel.debug) {
    return (msg) => {
        if (msg.level < level)
            return;
        let name = msg.name ? `[${msg.name}]` : "";
        switch (msg.level) {
            case LogLevel.info:
                console.info(`I|*   ${name}`, ...msg.messages);
                break;
            case LogLevel.warn:
                console.warn(`W|**  ${name}`, ...msg.messages);
                break;
            case LogLevel.error:
                console.error(`E|*** ${name}`, ...msg.messages);
                break;
            default:
                console.debug(`D|    ${name}`, ...msg.messages);
                break;
        }
    };
}
export function LoggerFactory(prefix = "", opt = {}) {
    var _a, _b, _c;
    let logHandlers = (opt === null || opt === void 0 ? void 0 : opt.handlers) || [LoggerConsoleHandler()];
    let logAssertLevel = LogLevel.warn;
    // let logLevel: LogLevel = 0
    function LoggerBaseFactory(name = "") {
        name = Logger._prefix + name;
        const emit = (msg) => {
            if (log.active === true) {
                if (msg.level >= Logger.level && msg.level >= log.level) {
                    if (Logger._isNamespaceAllowed(name)) {
                        for (let handler of logHandlers) {
                            if (handler)
                                handler(msg);
                        }
                    }
                }
            }
        };
        function log(...messages) {
            emit({
                name,
                messages,
                level: LogLevel.debug,
            });
        }
        log.active = true;
        log.level = LogLevel.debug;
        log.debug = function (...messages) {
            emit({
                name,
                messages,
                level: LogLevel.debug,
            });
        };
        log.info = function (...messages) {
            emit({
                name,
                messages,
                level: LogLevel.info,
            });
        };
        log.warn = function (...messages) {
            emit({
                name,
                messages,
                level: LogLevel.warn,
            });
        };
        log.error = function (...messages) {
            emit({
                name,
                messages,
                level: LogLevel.error,
            });
        };
        // fatal(...args: any[]) {
        //   console.error(...args)
        // },
        log.assert = function (cond, ...messages) {
            if (!cond) {
                if (typeof console !== undefined) {
                    if (console.assert) {
                        // https://developer.mozilla.org/de/docs/Web/API/Console/assert
                        console.assert(cond, ...messages);
                    }
                    else {
                        console.error(`Assert did fail with: ${cond}`, ...messages);
                    }
                }
                emit({
                    name,
                    messages: messages || [`Assert did fail with: ${cond}`],
                    level: logAssertLevel,
                });
                // try {
                //   if (typeof expect !== undefined) {
                //     expect(cond).toBeTruthy()
                //   }
                // } catch (err) {
                //   methods.warn(...args)
                // }
            }
        };
        log.assertEqual = function (value, expected, ...args) {
            let equal = deepEqual(value, expected);
            if (!equal) {
                log.assert(equal, `Assert did fail. Expected ${expected} got ${value}`, expected, value, ...args);
                // } else {
                //   methods.debug(`Passed equal`)
            }
        };
        log.assertNotEqual = function (value, expected, ...args) {
            let equal = deepEqual(value, expected);
            if (equal) {
                log.assert(equal, `Assert did fail. Expected ${expected} not to be equal with ${value}`, expected, value, ...args);
                // } else {
                //   methods.debug(`Passed not equal check`)
            }
        };
        log.extend = function (prefix) {
            return Logger.extend(name + ":" + prefix);
            // return Logger.extend(prefix)
        };
        // This is the trick, log is a function but also an object makes both valid:
        // const log = Logger(); log('test')
        // const {debug, info} = Logger(); info('test')
        return log;
    }
    function Logger(name = "") {
        // console.log("Logger with name", name, Logger._factory)
        return Logger._factory(name);
    }
    Logger.registerHandler = function (handler) {
        logHandlers.push(handler);
    };
    Logger._reject = (_a = opt === null || opt === void 0 ? void 0 : opt.reject) !== null && _a !== void 0 ? _a : [];
    Logger._accept = (_b = opt === null || opt === void 0 ? void 0 : opt.accept) !== null && _b !== void 0 ? _b : [];
    Logger.setFilter = function (namespaces) {
        Logger._reject = [];
        Logger._accept = [];
        if (namespaces && namespaces !== "*") {
            let i;
            const split = namespaces.split(/[\s,]+/);
            const len = split.length;
            for (i = 0; i < len; i++) {
                if (!split[i]) {
                    // ignore empty strings
                    continue;
                }
                namespaces = split[i].replace(/\*/g, ".*?");
                if (namespaces[0] === "-") {
                    Logger._reject.push(new RegExp("^" + namespaces.substr(1) + "$"));
                }
                else {
                    Logger._accept.push(new RegExp("^" + namespaces + "$"));
                }
            }
        }
    };
    Logger._isNamespaceAllowed = function (name) {
        if (Logger._reject.length === 0 && Logger._accept.length === 0) {
            return true;
        }
        let i, len;
        for (i = 0, len = Logger._reject.length; i < len; i++) {
            if (Logger._reject[i].test(name)) {
                return false;
            }
        }
        for (i = 0, len = Logger._accept.length; i < len; i++) {
            if (Logger._accept[i].test(name)) {
                return true;
            }
        }
        return false;
    };
    Logger._prefix = prefix;
    Logger.setPrefix = function (prefix) {
        Logger._prefix = prefix + (prefix.endsWith(":") ? "" : ":");
    };
    Logger.setHandlers = function (handlers = []) {
        logHandlers = [...handlers].filter((h) => typeof h === "function");
    };
    Logger.level = (_c = opt === null || opt === void 0 ? void 0 : opt.level) !== null && _c !== void 0 ? _c : LogLevel.debug;
    /** @deprecated */
    Logger.setLogLevel = function (level = 0) {
        Logger.level = level;
        // logLevel = level
    };
    Logger.extend = function (prefix) {
        if (Logger._prefix) {
            prefix = Logger._prefix + ":" + prefix;
        }
        if ((prefix === null || prefix === void 0 ? void 0 : prefix.length) > 0) {
            return LoggerFactory(prefix, {
                handlers: logHandlers,
                level: Logger.level,
                accept: Logger._accept,
                reject: Logger._reject,
            })();
        }
        throw new Error("Logger.extend needs a prefix with minimal length of 1");
    };
    Logger._factory = LoggerBaseFactory;
    Logger.setFactory = function (factory) {
        Logger._factory = factory;
    };
    return Logger;
}
export const Logger = LoggerFactory();
// console.log("Logger", Logger)
// export function LogMemoryHandler(msg: LogMessage) {}
//# sourceMappingURL=log.js.map